@@ -9,7 +9,8 @@ use TAP::Parser::Result       ();
 use TAP::Parser::Source       ();
 use TAP::Parser::Source::Perl ();
 use TAP::Parser::Iterator     ();
-use Carp                      ();
+
+use Carp qw( confess );
 
 @ISA = qw(TAP::Base);
 
@@ -19,11 +20,11 @@ TAP::Parser - Parse L<TAP|Test::Harness::TAP> output
 
 =head1 VERSION
 
-Version 3.09
+Version 3.11
 
 =cut
 
-$VERSION = '3.09';
+$VERSION = '3.11';
 
 my $DEFAULT_TAP_VERSION = 12;
 my $MAX_TAP_VERSION     = 13;
@@ -41,7 +42,6 @@ BEGIN {    # making accessors
         qw(
         _stream
         _spool
-        _grammar
         exec
         exit
         is_good_plan
@@ -58,28 +58,11 @@ BEGIN {    # making accessors
       )
     {
         no strict 'refs';
-
-        # another tiny performance hack
-        if ( $method =~ /^_/ ) {
-            *$method = sub {
-                my $self = shift;
-                return $self->{$method} unless @_;
-
-                # Trusted methods
-                unless ( ( ref $self ) =~ /^TAP::Parser/ ) {
-                    Carp::croak("$method() may not be set externally");
-                }
-
-                $self->{$method} = shift;
-            };
-        }
-        else {
-            *$method = sub {
-                my $self = shift;
-                return $self->{$method} unless @_;
-                $self->{$method} = shift;
-            };
-        }
+        *$method = sub {
+            my $self = shift;
+            return $self->{$method} unless @_;
+            $self->{$method} = shift;
+        };
     }
 }    # done making accessors
 
@@ -315,14 +298,15 @@ sub run {
 
         $self->SUPER::_initialize( \%args, \@legal_callback );
 
-        my $stream    = delete $args{stream};
-        my $tap       = delete $args{tap};
-        my $source    = delete $args{source};
-        my $exec      = delete $args{exec};
-        my $merge     = delete $args{merge};
-        my $spool     = delete $args{spool};
-        my $switches  = delete $args{switches};
-        my @test_args = @{ delete $args{test_args} || [] };
+        my $stream      = delete $args{stream};
+        my $tap         = delete $args{tap};
+        my $source      = delete $args{source};
+        my $exec        = delete $args{exec};
+        my $merge       = delete $args{merge};
+        my $spool       = delete $args{spool};
+        my $switches    = delete $args{switches};
+        my $ignore_exit = delete $args{ignore_exit};
+        my @test_args   = @{ delete $args{test_args} || [] };
 
         if ( 1 < grep {defined} $stream, $tap, $source, $exec ) {
             $self->_croak(
@@ -374,12 +358,8 @@ sub run {
         }
 
         $self->_stream($stream);
-        my $grammar = TAP::Parser::Grammar->new($stream);
-        $grammar->set_version( $self->version );
-        $self->_grammar($grammar);
         $self->_spool($spool);
-
-        $self->start_time( $self->get_time );
+        $self->ignore_exit($ignore_exit);
 
         return $self;
     }
@@ -411,6 +391,10 @@ examples of each, are as follows:
 
  1..42
 
+=item * Pragma
+
+ pragma +strict
+
 =item * Test
 
  ok 3 - We should start with some foobar!
@@ -521,6 +505,18 @@ If a SKIP directive is included with the plan, this method will return it.
 If a SKIP directive was included with the plan, this method will return the
 explanation, if any.
 
+=head2 C<pragma> methods
+
+ if ( $result->is_pragma ) { ... }
+
+If the above evaluates as true, the following methods will be available on the
+C<$result> object.
+
+=head3 C<pragmas>
+
+Returns a list of pragmas each of which is a + or - followed by the
+pragma name.
+ 
 =head2 C<commment> methods
 
  if ( $result->is_comment ) { ... }
@@ -782,6 +778,47 @@ This method lets you know which (or how many) tests had SKIP directives.
 
 sub skipped { @{ shift->{skipped} } }
 
+=head2 Pragmas
+
+=head3 C<pragma>
+
+Get or set a pragma. To get the state of a pragma:
+
+  if ( $p->pragma('strict') ) {
+      # be strict
+  }
+
+To set the state of a pragma:
+
+  $p->pragma('strict', 1); # enable strict mode
+
+=cut
+
+sub pragma {
+    my ( $self, $pragma ) = splice @_, 0, 2;
+
+    return $self->{pragma}->{$pragma} unless @_;
+
+    if ( my $state = shift ) {
+        $self->{pragma}->{$pragma} = 1;
+    }
+    else {
+        delete $self->{pragma}->{$pragma};
+    }
+
+    return;
+}
+
+=head3 C<pragmas>
+
+Get a list of all the currently enabled pragmas:
+
+  my @pragmas_enabled = $p->pragmas;
+
+=cut
+
+sub pragmas { sort keys %{ shift->{pragma} || {} } }
+
 =head2 Summary Results
 
 These results are "meta" information about the total results of an individual
@@ -861,8 +898,7 @@ sub has_problems {
     return
          $self->failed
       || $self->parse_errors
-      || $self->wait
-      || $self->exit;
+      || ( !$self->ignore_exit && ( $self->wait || $self->exit ) );
 }
 
 =head3 C<version>
@@ -888,6 +924,20 @@ Once the parser is done, this will return the wait status.  If the parser ran
 an executable, it returns the wait status of the executable.  Otherwise, this
 mererely returns the C<exit> status.
 
+=head2 C<ignore_exit>
+
+  $parser->ignore_exit(1);
+
+Tell the parser to ignore the exit status from the test when determining
+whether the test passed. Normally tests with non-zero exit status are
+considered to have failed even if all individual tests passed. In cases
+where it is not possible to control the exit value of the test script
+use this option to ignore it.
+
+=cut
+
+sub ignore_exit { shift->pragma( 'ignore_exit', @_ ) }
+
 =head3 C<parse_errors>
 
  my @errors = $parser->parse_errors; # the parser errors
@@ -965,14 +1015,33 @@ sub _make_state_table {
     my %state_globals = (
         comment => {},
         bailout => {},
+        yaml    => {},
         version => {
             act => sub {
-                my ($version) = @_;
                 $self->_add_error(
                     'If TAP version is present it must be the first line of output'
                 );
             },
         },
+        unknown => {
+            act => sub {
+                my $unk = shift;
+                if ( $self->pragma('strict') ) {
+                    $self->_add_error(
+                        'Unknown TAP token: "' . $unk->raw . '"' );
+                }
+            },
+        },
+        pragma => {
+            act => sub {
+                my ($pragma) = @_;
+                for my $pr ( $pragma->pragmas ) {
+                    if ( $pr =~ /^ ([-+])(\w+) $/x ) {
+                        $self->pragma( $2, $1 eq '+' );
+                    }
+                }
+            },
+        },
     );
 
     # Provides default elements for transitions
@@ -1039,9 +1108,7 @@ sub _make_state_table {
                   } => $number;
             },
         },
-        yaml => {
-            act => sub { },
-        },
+        yaml => { act => sub { }, },
     );
 
     # Each state contains a hash the keys of which match a token type. For
@@ -1125,7 +1192,7 @@ sub _make_state_table {
     );
 
     # Apply globals and defaults to state table
-    for my $name ( sort keys %states ) {
+    for my $name ( keys %states ) {
 
         # Merge with globals
         my $st = { %state_globals, %{ $states{$name} } };
@@ -1155,19 +1222,37 @@ determine the readiness of this parser.
 
 sub get_select_handles { shift->_stream->get_select_handles }
 
+sub _make_grammar_with_stream {
+    my ( $self, $stream ) = @_;
+    my $grammar = TAP::Parser::Grammar->new($stream);
+    $grammar->set_version( $self->version );
+    return $grammar;
+}
+
+sub _grammar {
+    my $self = shift;
+    if (@_) {
+        return $self->{_grammar} = shift;
+    }
+
+    return $self->{_grammar}
+      ||= $self->_make_grammar_with_stream( $self->_stream );
+}
+    
 sub _iter {
     my $self        = shift;
     my $stream      = $self->_stream;
-    my $spool       = $self->_spool;
     my $grammar     = $self->_grammar;
+    my $spool       = $self->_spool;
     my $state       = 'INIT';
     my $state_table = $self->_make_state_table;
 
+    $self->start_time( $self->get_time );
+
     # Make next_state closure
     my $next_state = sub {
         my $token = shift;
         my $type  = $token->type;
-        my $count = 1;
         TRANS: {
             my $state_spec = $state_table->{$state}
               or die "Illegal state: $state";
@@ -1184,6 +1269,9 @@ sub _iter {
                     $state = $goto;
                 }
             }
+            else {
+                confess("Unhandled token type: $type\n");
+            }
         }
         return $token;
     };
