@@ -1,7 +1,7 @@
 package TAP::Parser::Scheduler::Job;
 
 use strict;
-use warnings;
+use vars qw($VERSION);
 use Carp;
 
 =head1 NAME
@@ -10,11 +10,11 @@ TAP::Parser::Scheduler::Job - A single testing job.
 
 =head1 VERSION
 
-Version 3.51_01
+Version 3.11
 
 =cut
 
-our $VERSION = '3.51_01';
+$VERSION = '3.11';
 
 =head1 SYNOPSIS
 
@@ -31,11 +31,10 @@ Represents a single test 'job'.
 =head3 C<new>
 
     my $job = TAP::Parser::Scheduler::Job->new(
-        $filename, $description
+        $name, $desc 
     );
 
-Given the filename and description of a test as scalars, returns a new
-L<TAP::Parser::Scheduler::Job> object.
+Returns a new C<TAP::Parser::Scheduler::Job> object.
 
 =cut
 
@@ -44,18 +43,13 @@ sub new {
     return bless {
         filename    => $name,
         description => $desc,
-        @ctx ? ( context => \@ctx ) : (),
+        context     => \@ctx,
     }, $class;
 }
 
-=head2 Instance Methods
-
 =head3 C<on_finish>
 
-    $self->on_finish(\&method).
-
-Register a closure to be called when this job is destroyed. The callback
-will be passed the C<TAP::Parser::Scheduler::Job> object as it's only argument.
+Register a closure to be called when this job is destroyed.
 
 =cut
 
@@ -66,10 +60,7 @@ sub on_finish {
 
 =head3 C<finish>
 
-   $self->finish;
-
-Called when a job is complete to unlock it. If a callback has been registered
-with C<on_finish>, it calls it. Otherwise, it does nothing. 
+Called when a job is complete to unlock it.
 
 =cut
 
@@ -80,15 +71,6 @@ sub finish {
     }
 }
 
-=head2 Attributes
-
-  $self->filename;
-  $self->description;
-  $self->context;
-
-These are all "getters" which return the data set for these attributes during object construction.
-
-
 =head3 C<filename>
 
 =head3 C<description>
@@ -99,7 +81,7 @@ These are all "getters" which return the data set for these attributes during ob
 
 sub filename    { shift->{filename} }
 sub description { shift->{description} }
-sub context     { @{ shift->{context} || [] } }
+sub context     { @{ shift->{context} } }
 
 =head3 C<as_array_ref>
 
@@ -109,13 +91,11 @@ For backwards compatibility in callbacks.
 
 sub as_array_ref {
     my $self = shift;
-    return [ $self->filename, $self->description, $self->{context} ||= [] ];
+    return [ $self->filename, $self->description, $self->context ];
 }
 
 =head3 C<is_spinner>
 
-  $self->is_spinner;
-
 Returns false indicating that this is a real job rather than a
 'spinner'. Spinners are returned when the scheduler still has pending
 jobs but can't (because of locking) return one right now.
