@@ -1,7 +1,7 @@
 package TAP::Parser::Scheduler::Spinner;
 
 use strict;
-use warnings;
+use vars qw($VERSION);
 use Carp;
 
 =head1 NAME
@@ -10,11 +10,11 @@ TAP::Parser::Scheduler::Spinner - A no-op job.
 
 =head1 VERSION
 
-Version 3.51_01
+Version 3.11
 
 =cut
 
-our $VERSION = '3.51_01';
+$VERSION = '3.11';
 
 =head1 SYNOPSIS
 
@@ -34,14 +34,12 @@ return a real job.
 
     my $job = TAP::Parser::Scheduler::Spinner->new;
 
-Ignores any arguments and returns a new C<TAP::Parser::Scheduler::Spinner> object.
+Returns a new C<TAP::Parser::Scheduler::Spinner> object.
 
 =cut
 
 sub new { bless {}, shift }
 
-=head2 Instance Methods
-
 =head3 C<is_spinner>
 
 Returns true indicating that is a 'spinner' job. Spinners are returned
@@ -52,10 +50,4 @@ return one right now.
 
 sub is_spinner {1}
 
-=head1 SEE ALSO
-
-L<TAP::Parser::Scheduler>, L<TAP::Parser::Scheduler::Job>
-
-=cut
-
 1;
